@@ -87,6 +87,7 @@ MCMCconf <- setRefClass(
         controlDefaults     = 'ANY',
         unsampledNodes      = 'ANY',
         postPredSamplerDownstreamNodes = 'ANY',
+        allowData_global    = 'ANY',
         ##namedSamplerLabelMaker = 'ANY',  ## usage long since deprecated (Dec 2020)
         mvSamples1Conf      = 'ANY',
         mvSamples2Conf      = 'ANY'
@@ -165,12 +166,34 @@ print: A logical argument specifying whether to print the montiors and samplers.
             controlDefaults <<- list(...)
             ##namedSamplerLabelMaker <<- labelFunctionCreator('namedSampler')  ## usage long since deprecated (Dec 2020)
             for(i in seq_along(control))     controlDefaults[[names(control)[i]]] <<- control[[i]]
+
+            ## Conservatively insist that include any predictive nodes for CRP sampling.
+            if(!getNimbleOption('MCMCusePredictiveDependenciesInCalculations')) {
+                allDists <- unlist(lapply(model$modelDef$declInfo, `[[`, 'distributionName'))
+                nonNAs <- !is.na(allDists)
+                allDists <- allDists[nonNAs]
+                crpDists <- which(allDists == "dCRP")
+                if(length(crpDists)) {
+                    nodeExprs <- unlist(lapply(model$modelDef$declInfo, `[[`, 'targetNodeExpr'))[nonNAs]
+                    for(i in crpDists) {
+                        clusterNodeInfo <- findClusterNodes(model, deparse(nodeExprs[[i]]))
+                        allClusterNodesVec <- unlist(clusterNodeInfo$clusterNodes)
+                        if(length(allClusterNodesVec)) {
+                            clusterNodeDeps <- model$getDependencies(allClusterNodesVec, stochOnly = TRUE, self = FALSE)
+                            if(any(clusterNodeDeps %in% model$getNodeNames(predictiveOnly = TRUE)))
+                                stop("Discovered predictive node dependencies of dCRP node. This will generally be incompatible with NIMBLE's default MCMC configuration for CRP-based models. You can proceed with sampling by setting `nimbleOptions(MCMCusePredictiveDependenciesInCalculations = TRUE)` before setting up your MCMC.")
+                        }
+                    }
+                }
+            }
+            
             if(missing(nodes)) {
                 nodes <- model$getNodeNames(stochOnly = TRUE, includeData = FALSE)
                 # Check of all(model$isStoch(nodes)) is not needed in this case
-            } else {
-                if(is.null(nodes) || length(nodes)==0)     nodes <- character(0)
-            }
+            } else if(is.null(nodes) || length(nodes)==0) {
+                nodes <- character(0)
+            } else   nodes <- filterOutDataNodes(nodes)   ## configureMCMC *never* assigns samplers to data nodes
+            allowData_global <<- FALSE
             
             addDefaultSampler(nodes = nodes,
                               useConjugacy = useConjugacy,
@@ -193,7 +216,7 @@ print: A logical argument specifying whether to print the montiors and samplers.
             '
 For internal use.  Adds default MCMC samplers to the specified nodes.
 '
-            useNewConfigureMCMC <- isTRUE(nimbleOptions("useNewConfigureMCMC"))
+            useNewConfigureMCMC <- isTRUE(getNimbleOption("useNewConfigureMCMC"))
             
             controlDefaultsArg <- list(...)
             for(i in seq_along(control))     controlDefaultsArg[[names(control)[i]]] <- control[[i]]
@@ -352,7 +375,8 @@ For internal use.  Adds default MCMC samplers to the specified nodes.
                                 addConjugateSampler(conjugacyResult = conjugacyResult,
                                                     dynamicallyIndexed = model$modelDef$varInfo[[model$getVarNames(nodes=node)]]$anyDynamicallyIndexed);     next }
                         }
-                        if(nodeDist == 'dmulti')              { addSampler(target = node, type = 'RW_multinomial', control = controlDefaultsArg);     next }
+                        ##if(nodeDist == 'dmulti')              { addSampler(target = node, type = 'RW_multinomial', control = controlDefaultsArg);     next }
+                        if(nodeDist == 'dmulti')   stop('  [Error] Support for sampling multinomial distributions was removed from nimble due to incorrect sampling results.\n  Please contact nimble developers at the nimble-users google group or at nimble.stats@gmail.com to let them know this happened.  \n Thank you.', call. = FALSE)
                         if(nodeDist == 'ddirch')              { addSampler(target = node, type = 'RW_dirichlet',   control = controlDefaultsArg);     next }
                         if(nodeDist == 'dwish')               { addSampler(target = node, type = 'RW_wishart',     control = controlDefaultsArg);     next }
                         if(nodeDist == 'dinvwish')            { addSampler(target = node, type = 'RW_wishart',     control = controlDefaultsArg);     next }
@@ -481,19 +505,19 @@ For internal use.  Adds default MCMC samplers to the specified nodes.
                                 }
 
                                 samplers <- getSamplers(clusterNodes)
-                                removeSamplers(clusterNodes)
                                 for(i in seq_along(samplers)) {
                                     node <- samplers[[i]]$target
                                     ## getSamplers() returns samplers in order of configuration not in order of input.
                                     clusterID <- which(clusterNodes == node)
-                                    if(length(clusterID) != 1)
-                                        stop("Cannot determine wrapped sampler for cluster parameter ", node, ".")
-                                    controlCRP <- controlDefaultsArg
-                                    controlCRP$wrapped_type <- samplers[[i]]$name
-                                    controlCRP$wrapped_conf <- samplers[[i]]
-                                    controlCRP$dcrpNode <- dcrpNode[[k]]
-                                    controlCRP$clusterID <- clusterNodeInfo[[k]]$clusterIDs[[idx]][clusterID]
-                                    addSampler(target = node, type = 'CRP_cluster_wrapper', control = controlCRP)
+                                    if(length(clusterID) ==1) {  # Joint sampling on multiple nodes won't use wrapping.
+                                        removeSamplers(node)
+                                        controlCRP <- controlDefaultsArg
+                                        controlCRP$wrapped_type <- samplers[[i]]$name
+                                        controlCRP$wrapped_conf <- samplers[[i]]
+                                        controlCRP$dcrpNode <- dcrpNode[[k]]
+                                        controlCRP$clusterID <- clusterNodeInfo[[k]]$clusterIDs[[idx]][clusterID]
+                                        addSampler(target = node, type = 'CRP_cluster_wrapper', control = controlCRP)
+                                    }
                                 }
                             }
                         }
@@ -528,28 +552,30 @@ For internal use.  Adds default MCMC samplers to the specified nodes.
             addSampler(target = conjugacyResult$target, type = conjSamplerFunction, control = conjugacyResult$control, print = print, name = nameToPrint)
         },
         
-        addSampler = function(target = character(),    ## target argument is *not* expanded (unless expandTarget = TRUE)
+        addSampler = function(target = character(),    ## target argument is *not* expanded (unless targetByNode = TRUE)
                               type = 'RW',
                               control = list(),
                               print = NULL,        ## default value print is TRUE when default=TRUE, and FALSE otherwise
                               name,
-                              scalarComponents = FALSE,
-                              expandTarget = FALSE,
+                              targetByNode = FALSE,
+                              multivariateNodesAsScalars = getNimbleOption('MCMCmultivariateNodesAsScalars'),
+                              expandTarget,        ## renamed to 'targetByNode',               gives a deprecation message
+                              scalarComponents,    ## renamed to 'multivariateNodesAsScalars', gives a deprecation message
                               silent = FALSE,
                               default = FALSE,
                               useConjugacy = getNimbleOption('MCMCuseConjugacy'),
                               onlyRW = FALSE,
                               onlySlice = FALSE,
-                              multivariateNodesAsScalars = getNimbleOption('MCMCmultivariateNodesAsScalars'),
+                              allowData = FALSE,
                               ...) {
             '
 Adds a sampler to the list of samplers contained in the MCMCconf object.
 
 Arguments:
 
-target: The target node or nodes to be sampled.  This may be specified as a character vector of model node and/or variable names.  For univariate samplers, only a single target node should be provided (unless \'expandTarget\' is TRUE).  For multivariate samplers, one instance of the multivariate sampler will be assigned to all nodes specified.  Nodes are specified in combination with the \'expandTarget\' and \'scalarComponents\' arguments.  See details.
+target: The target node or nodes to be sampled.  This may be specified as a character vector of model node and/or variable names.  For univariate samplers, only a single target node should be provided (unless \'targetByNode\' is TRUE).  For multivariate samplers, one instance of the multivariate sampler will be assigned to all nodes specified.  Nodes are specified in combination with the \'targetByNode\' and \'multivariateNodesAsScalars\' arguments.
 
-type: When \'default\' is FALSE, specifies the type of sampler to add, specified as either a character string or a nimbleFunction object.  If the character argument type=\'newSamplerType\', then either newSamplerType or sampler_newSamplertype must correspond to a nimbleFunction (i.e. a function returned by nimbleFunction, not a specialized nimbleFunction).  Alternatively, the type argument may be provided as a nimbleFunction itself rather than its name.  In that case, the \'name\' argument may also be supplied to provide a meaningful name for this sampler.  The default value is \'RW\' which specifies scalar adaptive Metropolis-Hastings sampling with a normal proposal distribution. This default will result in an error if \'target\' specifies more than one target node (unless \'expandTarget\' is TRUE).  This argument is not used when the \'default\' argument is TRUE.
+type: When \'default\' is FALSE, specifies the type of sampler to add, specified as either a character string or a nimbleFunction object.  If the character argument type=\'newSamplerType\', then either newSamplerType or sampler_newSamplerType must correspond to a nimbleFunction (i.e. a function returned by nimbleFunction, not a specialized nimbleFunction).  Alternatively, the type argument may be provided as a nimbleFunction itself rather than its name.  In that case, the \'name\' argument may also be supplied to provide a meaningful name for this sampler.  The default value is \'RW\' which specifies scalar adaptive Metropolis-Hastings sampling with a normal proposal distribution. This default will result in an error if \'target\' specifies more than one target node (unless \'targetByNode\' is TRUE).  This argument is not used when the \'default\' argument is TRUE.
 
 control: An optional list of control arguments to sampler functions.  These will override those specified in the control list argument to configureMCMC.  If a control list is provided, the elements will be provided to all sampler functions which utilize the named elements given. For example, the standard Metropolis-Hastings random walk sampler (sampler_RW) utilizes control list elements \'adaptive\', \'adaptInterval\', \'scale\'. The default values for control list arguments for samplers (if not otherwise provided as an argument to configureMCMC or addSampler) are contained in the setup code of each sampling algorithm.
 
@@ -557,9 +583,9 @@ print: Logical argument, specifying whether to print the details of newly added
 
 name: Optional character string name for the sampler, which is used by the printSamplers method.  If \'name\' is not provided, the \'type\' argument is used to generate the sampler name.
 
-scalarComponents: Logical argument, with default value FALSE.  When \'expandTarget\' is TRUE and therefore \'target\' undergoes expansion via \'expandNodeNames\', this argument is passed as the \'returnScalarComponents\' argument to \'expandNodeNames\'.  This has the effect of returning the constituent scalar components which make up any multivariate nodes in \'target\', in which case a separate instance of the specified sampler type will be added to each scalar component.  This argument is only used when \'expandTarget\' is TRUE.  See details.
+targetByNode: Logical argument, with default FALSE.  This arguments controls whether separate instances of the specified sampler \'type\' should be assigned to each node contained in \'target\'.  When FALSE, a single instance of sampler \'type\' is assigned to operate on \'target\'.  When TRUE, potentially multiple instances of sampler \'type\' will be added to the MCMC configuration, operating on the distinct nodes which compose \'target\'.  For example, if \'target\' is a vector of distinct node names, then a separate sampler will be assigned to each node in this vector.  If \'target\' is a model variable which itself is comprised of multiple distinct nodes, then a separate sampler is assigned to each node composing the \'target\' variable. Additional control of the handling of multivariate nodes is provided using the \'multivariateNodesAsScalars\' argument.
 
-expandTarget: Logical argument, indicating whether to expand the \'target\' argument using \'expandNodeNames\' into the underlying constitutent model nodes.  These may include univariate and/or multivariate nodes, although additional control is provided using the \'scalarComponents\' argument.  When \'expandTarget\' is TRUE and this expansion to individual nodes occurs, samplers will be assigned independently to each (univariate or multivariate) node component of \'target\'.  That is, distinct instances of the specified sampler type will be assigned to each of the nodes which comprise \'target\'.  If \'target\' is comprised of multiple univariate nodes, then a univariate sampler should be specified, and similarly when \'target\' expands to one or more multivariate nodes, then a multivariate sampler should be specified.
+multivariateNodesAsScalars: Logical argument, with default value FALSE.  This argument is used in two ways.  Functionally, both uses result in separate instances of samplers being added to the scalar components which compose multivariate nodes. See details below.
 
 silent: Logical argument, specifying whether to print warning messages when assigning samplers.
 
@@ -571,21 +597,26 @@ onlyRW: Logical argument, with default value FALSE.  If specified as TRUE, then
 
 onlySlice: Logical argument, with default value FALSE.  If specified as TRUE, then a slice sampler is assigned for all non-terminal nodes. Terminal nodes are still assigned a posterior_predictive sampler.  This argument is only used when the \'default\' argument is TRUE.
 
-multivariateNodesAsScalars: Logical argument, with default value FALSE.  If specified as TRUE, then non-terminal multivariate stochastic nodes will have scalar samplers assigned to each of the scalar components of the multivariate node.  The default value of FALSE results in a single block sampler assigned to the entire multivariate node.  Note, multivariate nodes appearing in conjugate relationships will be assigned the corresponding conjugate sampler (provided \'useConjugacy\' is TRUE), regardless of the value of this argument.  This argument is only used when the \'default\' argument is TRUE.
+allowData: Logical argument, with default value FALSE.  When FALSE, samplers will not be assigned to operate on data nodes, even if data nodes are included in \'target\'.  When TRUE, samplers will be assigned to \'target\' without regard to whether nodes are designated as data.
 
 ...: Additional named arguments passed through ... will be used as additional control list elements.
 
 Details:
 
-Samplers will be assigned to nodes specified by the \'target\' argument.  The \'target\' argument does not undergo expansion, unless \'expandTarget\' is TRUE, in which case \'target\' is expanded to the underlying (univariate or multivariate) nodes, and a distinct sampler is added to each.  When \'expandTarget\' is TRUE and \'scalarComponents\' is TRUE, then any multivariate nodes are further decomponsed into their underlying scalar components, and a separate sampler is added to each.
-
-Samplers are added added to the end of the list of samplers for this MCMCconf object, and do not replace any exisiting samplers.  Samplers are removed using the removeSamplers method.
+Samplers are added to the end of the list of samplers for this MCMCconf object, and do not replace any existing samplers.  Samplers are removed using the removeSamplers method.
 
 Invisibly returns a list of the current sampler configurations, which are samplerConf reference class objects.
+
+\'multivariateNodesAsScalars\' has two usages. The first usage occurs when \'targetByNode\' is TRUE and therefore separate instances of sampler \'type\' are assigned to each node which compose \'target\'.  In this first usage, this argument controls how multivariate nodes (those included in the \'target\') are handled.  If FALSE, any multivariate nodes in \'target\' have a single instance of sampler \'type\' assigned.  If TRUE, any multivariate nodes appearing in \'target\' are themselves decomposed into their scalar elements, and a separate instance of sampler \'type\' is assigned to operate on each scalar element.
+
+The second usage of \'multivariateNodesAsScalars\' occurs when \'default\' is TRUE, and therefore samplers are assigned according to the default logic of configureMCMC, which is further controlled by the arguments \'useConjugacy\', \'onlyRW\', \'onlySlice\' and \'multivariateNodesAsScalars\'.  In this second usage, if \'multivariateNodesAsScalars\' is TRUE, then multivariate nodes will be decomposed into their scalar components, and separate samplers assigned to each scalar element.  Note, however, that multivariate nodes appearing in conjugate relationships will still be assigned the corresponding conjugate sampler (provided \'useConjugacy\' is TRUE), regardless of the value of this argument.  If \'multivariateNodesAsScalars\' is FALSE, then a single multivarate sampler will be assigned to update each multivariate node.  The default value of this argument can be controlled using the nimble option \'MCMCmultivariateNodesAsScalars\'.
 '
             if(length(target) == 0)   return(invisible(samplerConfs))
 
+            if(!(isTRUE(allowData) || isFALSE(allowData)))   stop('allowData argument can only be TRUE or FALSE', call. = FALSE)
+
             if(default) {
+                allowData_global <<- allowData
                 addDefaultSampler(nodes = target,
                                   control = control,
                                   useConjugacy = useConjugacy,
@@ -594,10 +625,14 @@ Invisibly returns a list of the current sampler configurations, which are sample
                                   multivariateNodesAsScalars = multivariateNodesAsScalars,
                                   print = if(is.null(print)) TRUE else print,   ## default of print is TRUE when adding default sampler
                                   ...)
+                allowData_global <<- FALSE
                 return(invisible(samplerConfs))
             }
 
             print <- if(is.null(print)) FALSE else print                        ## default of print is FALSE otherwise
+
+            if(!missing(expandTarget))       { messageIfVerbose('  [Warning] `expandTarget` argument name has been deprecated; please use `targetByNode.');                     targetByNode               <- expandTarget     }
+            if(!missing(scalarComponents))   { messageIfVerbose('  [Warning] `scalarComponents` argument name has been deprecated; please use `multivariateNodesAsScalars.');   multivariateNodesAsScalars <- scalarComponents }
             
             nameProvided <- !missing(name)
             if(is.character(type)) {
@@ -647,7 +682,14 @@ Invisibly returns a list of the current sampler configurations, which are sample
             if(!is.character(thisSamplerName)) stop('sampler name should be a character string')
             if(!is.function(samplerFunction)) stop('sampler type does not specify a function')
 
-            if(!(all(model$isStoch(target)))) { warning(paste0('No sampler assigned to non-stochastic node: ', paste0(target,collapse=', '))); return(invisible(samplerConfs)) }   ## ensure all target node(s) are stochastic
+            ## ensure all target node(s) are stochastic
+            if(!(all(model$isStoch(target)))) {
+                ## however, allow assignment of prior_samples sampler to RHS-only nodes
+                if(!(thisSamplerName == 'prior_samples' && all(model$getNodeType(target) == 'RHSonly'))) {
+                    warning(paste0('No sampler assigned to non-stochastic node: ', paste0(target,collapse=', ')))
+                    return(invisible(samplerConfs))
+                }
+            }
 
             ##libraryTag <- if(nameProvided) namedSamplerLabelMaker() else thisSamplerName   ## unique tag for each 'named' sampler, internal use only  ## usage long since deprecated (Dec 2020)
             ##if(is.null(controlNamesLibrary[[libraryTag]]))   controlNamesLibrary[[libraryTag]] <<- mcmc_findControlListNamesInCode(samplerFunction)   ## populate control names library
@@ -655,30 +697,41 @@ Invisibly returns a list of the current sampler configurations, which are sample
             controlArgs <- c(control, list(...))
             thisControlList <- mcmc_generateControlListArgument(control=controlArgs, controlDefaults=controlDefaults)  ## should name arguments
             
-            if(!expandTarget) {
-                ## no node expansion takes place when expandTarget is FALSE
-                addSamplerOne(thisSamplerName, samplerFunction, target, thisControlList, print)
+            if(!targetByNode) {
+                ## when targetByNode = FALSE,
+                ## no node expansion takes place
+                addOneSampler(thisSamplerName, samplerFunction, target, thisControlList, allowData, print)
             } else {
+                ## when targetByNode = TRUE,
                 ## assign sampler type to each component of target after expanding node names,
-                ## also using scalarComponents argument here, to control the node expansion
-                targetExpanded <- model$expandNodeNames(target, returnScalarComponents = scalarComponents, sort = TRUE)
+                ## also using multivariateNodesAsScalars argument here, to control the node expansion
+                targetExpanded <- model$expandNodeNames(target, returnScalarComponents = multivariateNodesAsScalars, sort = TRUE)
                 for(i in seq_along(targetExpanded)) {
-                    addSamplerOne(thisSamplerName, samplerFunction, targetExpanded[i], thisControlList, print)
+                    addOneSampler(thisSamplerName, samplerFunction, targetExpanded[i], thisControlList, allowData, print)
                 }
             }
             
             return(invisible(samplerConfs))
         },
 
-        addSamplerOne = function(thisSamplerName, samplerFunction, targetOne, thisControlList, print) {
+        addOneSampler = function(thisSamplerName, samplerFunction, targetOne, thisControlList, allowData, print) {
             '
 For internal use only
 '
+            if(!allowData && !allowData_global) {
+                if(all(model$isData(targetOne)))   return()
+                if(any(model$isData(targetOne)))   targetOne <- filterOutDataNodes(targetOne)
+            }
             newSamplerInd <- length(samplerConfs) + 1
             samplerConfs[[newSamplerInd]] <<- samplerConf(name=thisSamplerName, samplerFunction=samplerFunction, target=targetOne, control=thisControlList, model=model)
             samplerExecutionOrder <<- c(samplerExecutionOrder, newSamplerInd)
             if(print) printSamplers(newSamplerInd)
         },
+
+        filterOutDataNodes = function(nodes) {
+            nodes <- model$expandNodeNames(nodes)
+            return(nodes[!model$isData(nodes)])
+        },
         
         removeSamplers = function(..., ind, print = FALSE) {
             '
@@ -883,7 +936,7 @@ byType: A logical argument, specifying whether the nodes being sampled should be
                         theseNodesNotIndexed <- theseNodes[!isIndexedVector]
                         if(length(theseNodesNotIndexed)) {
                             if(length(theseNodesNotIndexed) == 1) cat(paste0(indent, theseNodesNotIndexed))
-                            if(length(theseNodesNotIndexed) >  1 && length(unique(theseNodesNotIndexed)) > 1) stop('something wrong with Daniel\'s understanding', call. = FALSE)
+                            if(length(theseNodesNotIndexed) >  1 && length(unique(theseNodesNotIndexed)) > 1) stop('internal error in printSamplersByType method', call. = FALSE)
                             if(length(theseNodesNotIndexed) >  1) cat(paste0(indent, theseNodesNotIndexed[1], '  (', length(theseNodesNotIndexed), ')'))
                             cat('\n')
                         }
@@ -1000,7 +1053,9 @@ Arguments:
 
 print: A logical argument specifying whether to print all current monitors (default TRUE).
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             
             if(isMvSamplesReady(ind)){
@@ -1025,7 +1080,7 @@ Details: See the initialize() function
             nl_checkVarNamesInModel(model, vars)
             if(ind == 1)     monitors  <<- sort(unique(c(monitors,  vars)))
             if(ind == 2)     monitors2 <<- sort(unique(c(monitors2, vars)))
-            if(print && nimbleOptions('verbose')) printMonitors()
+            if(print && getNimbleOption('verbose')) printMonitors()
             return(invisible(NULL))
         },
 
@@ -1039,7 +1094,9 @@ Arguments:
 
 print: A logical argument specifying whether to print all current monitors (default TRUE).
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             addMonitors(..., ind = 2, print = print)
         },
@@ -1054,7 +1111,9 @@ Arguments:
 
 print: A logical argument specifying whether to print all current monitors (default TRUE).
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             if(ind == 1)   monitors  <<- character()
             if(ind == 2)   monitors2 <<- character()
@@ -1071,7 +1130,9 @@ Arguments:
 
 print: A logical argument specifying whether to print all current monitors (default TRUE).
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             setMonitors(..., ind = 2, print = print)
         },
@@ -1080,7 +1141,9 @@ Details: See the initialize() function
             '
 Resets the current monitors and monitors2 lists to nothing.
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             monitors  <<- character()
             monitors2 <<- character()
@@ -1099,7 +1162,9 @@ Details: See the initialize() function
             '
 Prints all current monitors and monitors2
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             if(length(monitors)  > 0)   cat(paste0('thin = ',  thin,  ': ', paste0(monitors,  collapse = ', '), '\n'))
             if(length(monitors2) > 0)   cat(paste0('thin2 = ', thin2, ': ', paste0(monitors2, collapse = ', '), '\n'))
@@ -1109,7 +1174,9 @@ Details: See the initialize() function
             '
 Returns a character vector of the current monitors
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             return(monitors)
         },
@@ -1118,7 +1185,9 @@ Details: See the initialize() function
             '
 Returns a character vector of the current monitors2
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             return(monitors2)
         },
@@ -1133,13 +1202,15 @@ thin: The new value for the thinning interval \'thin\'.
 
 print: A logical argument specifying whether to print all current monitors (default TRUE).
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             if(thin < 1)              stop('cannot use thin < 1', call. = FALSE)
             if(thin != floor(thin))   stop('cannot use non-integer thin', call. = FALSE)
             if(ind == 1)   thin  <<- thin
             if(ind == 2)   thin2 <<- thin
-            if(print && nimbleOptions('verbose')) printMonitors()
+            if(print && getNimbleOption('verbose')) printMonitors()
             return(invisible(NULL))
         },
         setThin2 = function(thin2, print = TRUE) {
@@ -1152,7 +1223,9 @@ thin2: The new value for the thinning interval \'thin2\'.
 
 print: A logical argument specifying whether to print all current monitors (default TRUE).
 
-Details: See the initialize() function
+Details:
+
+See the initialize() function
             '
             setThin(thin = thin2, print = print, ind = 2)
         },
@@ -1213,15 +1286,18 @@ Details: See the initialize() function
             return(unsampledNodes)
         },
         
-        warnUnsampledNodes = function() {
+        warnUnsampledNodes = function(includeConfGetUnsampledNodes = TRUE) {
             if(length(unsampledNodes)) {
                 numUnsampled <- length(unsampledNodes)
                 sTag <- if(numUnsampled > 1) 's' else ''
-                messageIfVerbose('  [Warning] No samplers assigned for ', numUnsampled, ' node', sTag, ', use conf$getUnsampledNodes() for node name', sTag, '.')
+                msg <- paste0('  [Warning] No samplers assigned for ', numUnsampled, ' node', sTag)
+                if(includeConfGetUnsampledNodes)   msg <- paste0(msg, ', use conf$getUnsampledNodes() for node name', sTag)
+                msg <- paste0(msg, '.')
+                messageIfVerbose(msg)
             }
         },
 
-        printComments = function() {
+        printComments = function(...) {
             setUnsampledNodes()
             anyComments <-
                 length(postPredSamplerDownstreamNodes) ||
@@ -1229,16 +1305,16 @@ Details: See the initialize() function
             if(anyComments) {
                 cat('===== Comments =====\n')
                 if(length(postPredSamplerDownstreamNodes))   message('  [Note] Additional downstream predictive nodes are also being sampled by posterior_predictive sampler.')
-                if(getNimbleOption('MCMCwarnUnsampledStochasticNodes'))   warnUnsampledNodes()
+                if(getNimbleOption('MCMCwarnUnsampledStochasticNodes'))   warnUnsampledNodes(...)
             }
         },
 
-        show = function() {
+        show = function(...) {
             cat('===== Monitors =====\n')
             printMonitors()
             cat('===== Samplers =====\n')
             if(length(samplerConfs)) printSamplers(byType = TRUE) else cat('(no samplers assigned)\n')
-            printComments()
+            printComments(...)
         }
     )
 )
