@@ -312,7 +312,7 @@ cppNimbleFunctionClass <- setRefClass('cppNimbleFunctionClass',
                                                   ## if(static) {
                                                   ##     message("add_deriv_function support for static is not written.")
                                                   ## } else {
-                                                  if(isTRUE(nimbleOptions("useADreconfigure"))) {
+                                                  if(isTRUE(getNimbleOption("useADreconfigure"))) {
                                                       newFunName <- paste0(funName, '_deriv_')
                                                       argTransName <- paste0(funName, '_ADargumentTransfer2_')
                                                       functionDefs[[newFunName]] <<- make_deriv_function(regularFun,
@@ -584,7 +584,7 @@ cppNimbleFunctionClass <- setRefClass('cppNimbleFunctionClass',
                                                       addInheritance(baseClassName)
                                                       addAncestors('NamedObjects')
                                                   }
-                                                  handleDerivs <- isTRUE(nimbleOptions("enableDerivs")) &&
+                                                  handleDerivs <- isTRUE(getNimbleOption("enableDerivs")) &&
                                                       length(environment(nfProc$nfGenerator)$buildDerivs) > 0
                                                   if(handleDerivs) {
                                                       constructorCode <- addADclassContent() ## Might generate code to insert into constructor, which is built later
@@ -645,7 +645,8 @@ cppNimbleFunctionClass <- setRefClass('cppNimbleFunctionClass',
 ## The next block of code has the initial setup for an AST processing stage
 ## to make modifications for AD based on context etc.
 modifyForAD_handlers <- c(list(
-    pow = 'modifyForAD_issuePowWarning',
+    `[` = 'modifyForAD_indexingBracket',
+    # pow = 'modifyForAD_issuePowWarning',
     eigenBlock = 'modifyForAD_eigenBlock',
     calculate = 'modifyForAD_calculate',
     getValues = 'modifyForAD_getSetValues',
@@ -658,23 +659,32 @@ modifyForAD_handlers <- c(list(
     `*` = 'modifyForAD_matmult',
     eigInverse = 'modifyForAD_matinverse',
     MAKE_FIXED_VECTOR = 'modifyForAD_MAKE_FIXED_VECTOR'),
+  #  makeCallList(assignmentOperators, 'modifyForAD_assign'), # Handled manually inside modifyForAD_recurseForAD
     makeCallList(recyclingRuleOperatorsAD, 'modifyForAD_RecyclingRule'),
     makeCallList(c('EIGEN_FS', 'EIGEN_BS', 'EIGEN_SOLVE', 'EIGEN_CHOL', 'nimNewMatrixD', 'nimCd',
                    'nimRepd','nimDiagonalD', 'nimNonseqIndexedd'),
                  'modifyForAD_prependNimDerivs'))
 
 exprClasses_modifyForAD <- function(code, symTab,
-                                    workEnv = list2env(list(wrap_in_value = FALSE))) {
+                                    workEnv = list2env(list(wrap_in_value = FALSE,
+                                                            .anyAD = FALSE))) {
   if(code$isName) {
+    symObj <- symTab$getSymbolObject(code$name, inherits = TRUE)
+    # Check if the symbol is TYPE_ or CppAD::AD or a NimArr of TYPE_ or CppAD::AD
+    symIsAD <- !is.null(symObj) && (identical(symObj$baseType, "TYPE_") ||
+                                    identical(symObj$baseType, "CppAD::AD") ||
+                                    identical(symObj$baseType, "NimArr") && (identical(symObj$templateArgs[[2]], "TYPE_") ||
+                                                                             (inherits(symObj$templateArgs[[2]], "cppVarFull") &&
+                                                                              identical(symObj$templateArgs[[2]]$baseType, "CppAD::AD"))))
+    if(isTRUE(symIsAD)) workEnv$.anyAD <- TRUE
+
+    # Is this "wrap_in_value" step ever used or is it cruft?
     if(!isTRUE(workEnv[['wrap_in_value']]))
       return(invisible())
     else {
-      symObj <- symTab$getSymbolObject(code$name)
-      if(!is.null(symObj)) {
-        if(identical(symObj$baseType, "CppAD::AD")) {
+      if(isTRUE(symIsAD)) ## N.B. Previously this came from symTab$getSymbolObject(code$name, inherits = FALSE)
+        if(identical(symObj$baseType, "CppAD::AD")) # What objects get this type, rather than TYPE_
           insertExprClassLayer(code$caller, code$callerArgID, 'Value') ## It is ok to leave some fields (type, sizeExpr) unpopulated at this late processing stage
-        }
-      }
     }
   }
   if(code$isCall) {
@@ -685,14 +695,17 @@ exprClasses_modifyForAD <- function(code, symTab,
     if(workEnv$RsymTab$symbolExists(code$name, TRUE)) ## Could be a nimbleFunction class method
       modifyForAD_recordable(code, symTab, workEnv)
     else {
-      if(!is.null(code$aux)) {
-        if(isTRUE(code$aux[['buildDerivs']])) ## This is added in sizeRCfunction
-          modifyForAD_RCfunction(code, symTab, workEnv)
-      }
+      modifyForAD_RCfunction(code, symTab, workEnv)
     }
   }
   if(is.null(code$caller)) { ## It is the top level `{`
     if(identical(code$name, "{")) {
+      if(!isTRUE(workEnv$.inModel)) {
+        if(isTRUE(workEnv$.illegalADcall)) {
+          allMsgs <- paste(workEnv$.illegalADmsgs, collapse="\n")
+          stop(paste("Problem(s) creating derivative code.\n", allMsgs))
+        }
+      }
       if(isTRUE(workEnv[['hasCalculate']])) {
         if(is.null( workEnv[['nodeFxnVector_name']] ) )
           stop("While setting up C++ AD code: calculate found but nodeFxnVector_name not set.")
@@ -729,21 +742,41 @@ recurse_modifyForAD <- function(code, symTab, workEnv) {
   if(is.list(code$aux))
     if(isTRUE(code$aux$.avoidAD))
       return(invisible(NULL))
+  anyAD_on_input <- workEnv$.anyAD
+  anyAD_thisCall <- FALSE
+  assignment <- isTRUE(any(code$name == assignmentOperators))
   for(i in seq_along(code$args)) {
     if(inherits(code$args[[i]], 'exprClass')) {
+      if(assignment) workEnv$onLHS <- i == 1
+      workEnv$.anyAD <- FALSE
       exprClasses_modifyForAD(code$args[[i]], symTab, workEnv)
+      anyAD_thisCall <- anyAD_thisCall || isTRUE(workEnv$.anyAD)
+      if(assignment) workEnv$onLHS <- FALSE
     }
   }
+  workEnv$.anyAD <- anyAD_thisCall
   invisible(NULL)
 }
 
-modifyForAD_issuePowWarning <- function(code, symTab, workEnv) {
-  message("   [Note] Operator `pow` may cause derivative problems with negative arguments. If the exponent is guaranteed to be an integer, use `pow_int` instead.", call. = FALSE)
+modifyForAD_indexingBracket <- function(code, symTab, workEnv) {
+  if(!isTRUE(workEnv$.anyAD)) return(invisible(NULL))
+  if(is.null(code$type)) return(invisible(NULL)) # arises from constructions like setMap that lack type annotation
+  if(length(code$args) > 4) return(invisible(NULL)) # 4D or higher is not handled, assumed to be static indexing
+  newName <- "stoch_ind_get"
+  if(isTRUE(workEnv$onLHS))
+    if(code$caller$name %in% assignmentOperators)
+      newName <- "stoch_ind_set"
+  code$name <- newName
   invisible(NULL)
 }
 
+## modifyForAD_issuePowWarning <- function(code, symTab, workEnv) {
+##   message("   [Note] Operator `pow` may cause derivative problems with negative arguments. If the exponent is guaranteed to be an integer, use `pow_int` instead.")
+##   invisible(NULL)
+## }
+
 modifyForAD_matmult <- function(code, symTab, workEnv) {
-  if(isTRUE(nimbleOptions("useADmatMultAtomic")))
+  if(isTRUE(getNimbleOption("useADmatMultAtomic")))
     if(length(code$args)==2)  ## something is wrong if there are not 2 args
       if(!(isEigScalar(code$args[[1]]) || isEigScalar(code$args[[2]]))) ## are both non-scalar?
         code$name <- 'nimDerivs_matmult'
@@ -751,7 +784,7 @@ modifyForAD_matmult <- function(code, symTab, workEnv) {
 }
 
 modifyForAD_matinverse <- function(code, symTab, workEnv) {
-  if(isTRUE(nimbleOptions("useADmatInverseAtomic")))
+  if(isTRUE(getNimbleOption("useADmatInverseAtomic")))
     code$name <- 'nimDerivs_matinverse'
   invisible(NULL)
 }
@@ -840,10 +873,10 @@ modifyForAD_prependNimDerivs <- function(code, symTab, workEnv) {
   origName <- code$name
   atomic <- TRUE
   if(origName == 'EIGEN_FS' | origName == 'EIGEN_BS')
-    if(!isTRUE(nimbleOptions("useADsolveAtomic")))
+    if(!isTRUE(getNimbleOption("useADsolveAtomic")))
         atomic <- FALSE
   if(origName == "EIGEN_CHOL")
-    if(!isTRUE(nimbleOptions("useADcholAtomic")))
+    if(!isTRUE(getNimbleOption("useADcholAtomic")))
         atomic <- FALSE
   if(atomic)
     code$name <- paste0("nimDerivs_", origName)
@@ -853,7 +886,59 @@ modifyForAD_prependNimDerivs <- function(code, symTab, workEnv) {
 }
 
 modifyForAD_RCfunction <- function(code, symTab, workEnv) {
-  code$name <- paste0(code$name, "< CppAD::AD<double> >")
+  # If there is no AD happening, skip everything
+  if(isTRUE(workEnv$.anyAD)) {
+    lacksADsupport <- any(code$name == callsNotAllowedInAD)
+    # If there is lack of AD support due to disallowed keyword, skip to error handling
+    if(!lacksADsupport) {
+      isRCwithoutAD <- FALSE
+      # If there is code$aux, this is a potential AD call
+      if(!is.null(code$aux)) {
+        buildDerivs <- code$aux[['buildDerivs']]
+        # If code$aux$buildDerivs is set, this is a potential AD call
+        if(!is.null(buildDerivs)) {
+          # If code$aux$buildDerivs is TRUE, insert the template argument
+          if(isTRUE(buildDerivs)) {
+            code$name <- paste0(code$name, "< CppAD::AD<double> >")
+          } else {
+            # If code$aux$buildDerivs is not TRUE, this is an RC function without AD support
+            isRCwithoutAD <- TRUE
+          }
+        }
+      }
+      # So far lacksADsupport must be FALSE. Now we update it to include case isRCwithoutAD
+      lacksADsupport <- isRCwithoutAD
+    }
+    if(lacksADsupport) {
+      workEnv$.illegalADcall <- TRUE
+      if(!is.character(workEnv$.illegalADmsgs)) workEnv$.illegalADmsgs <- character()
+      workEnv$.illegalADmsgs <- c(workEnv$.illegalADmsgs,
+                                  exprClassProcessingErrorMsg(
+                                    code,
+                                    "A function that might be intended for derivative tracking does not have buildDerivs set to enable that."
+                                  ))
+    }
+  }
+  ## if(isTRUE(workEnv$.anyAD)) {
+  ##   isRCwithoutAD <- (!is.null(code$aux)) && (!isTRUE(code$aux[['buildDerivs']]))
+  ##   isRCwithAD <- (!is.null(code$aux)) && (isTRUE(code$aux[['buildDerivs']]))
+  ##   lacksADsupport <- isRCwithoutAD ||
+  ##                      any(code$name == callsNotAllowedInAD)
+  ##   supportsAD <- !lacksADsupport
+  ##   if(supportsAD) {
+  ##     if(isRCwithAD)
+  ##       code$name <- paste0(code$name, "< CppAD::AD<double> >")
+  ##   } else {
+  ##     browser()
+  ##     workEnv$.illegalADcall <- TRUE
+  ##     if(!is.character(workEnv$.illegalADmsgs)) workEnv$.illegalADmsgs <- character()
+  ##     workEnv$.illegalADmsgs <- c(workEnv$.illegalADmsgs,
+  ##                                 exprClassProcessingErrorMsg(
+  ##                                   code,
+  ##                                   "A function that might be intended for derivative tracking does not have buildDerivs set to enable that."
+  ##                                 ))
+  ##   }
+  ## }
   invisible(NULL)
 }
 
@@ -895,14 +980,35 @@ modifyForAD_chainedCall <- function(code, symTab, workEnv) {
   invisible(NULL)
 }
 
+# This is when the current method (which has buildDerivs set, else we wouldn't be here)
+# has a call to another method. We assume that method should also have buildDerivs set
+# any thus we should append the "_AD2_" on its name and throw an error if it doesn't
+# have buildDerivs set.
+# However, if no arguments seem to involve AD, we leave the call unmodified.
 modifyForAD_recordable <- function(code, symTab, workEnv) {
   symObj <- workEnv$RsymTab$getSymbolObject(code$name, TRUE)
   buildDerivs <- symObj$nfMethodRCobj$buildDerivs
-  if(!is.null(buildDerivs))
-    if(!isFALSE(buildDerivs)) {
+  anyAD <- workEnv$.anyAD
+  callSupportsAD <- (!is.null(buildDerivs)) && (!isFALSE(buildDerivs))
+  if(anyAD) {
+    if(callSupportsAD) {
       code$name <- paste0(code$name, "_AD2_")
       setArg(code, length(code$args) + 1, RparseTree2ExprClasses(quote(recordingInfo_)))
+    } else {
+      workEnv$.illegalADcall <- TRUE
+      if(!is.character(workEnv$.illegalADmsgs)) workEnv$.illegalADmsgs <- character()
+      workEnv$.illegalADmsgs <- c(workEnv$.illegalADmsgs,
+                                  exprClassProcessingErrorMsg(
+                                    code,
+                                    "A function that might be intended for derivative tracking does not have buildDerivs set to enable that."
+                                  ))
     }
+  }
+  ## if(!is.null(buildDerivs))
+  ##   if(!isFALSE(buildDerivs)) {
+  ##     code$name <- paste0(code$name, "_AD2_")
+  ##     setArg(code, length(code$args) + 1, RparseTree2ExprClasses(quote(recordingInfo_)))
+  ##   }
   invisible(NULL)
 }
 
@@ -950,49 +1056,100 @@ modifyForAD_calculate <- function(code, symTab, workEnv) {
 }
 
 updateADproxyModelMethods <- function(.self) {
-    ## Update return type and names of functions like dnorm -> nimDerivs_dnorm
-    functionNames <- names(.self$functionDefs)
-    ADproxyModel_functionNames <- functionNames[ grepl("_ADproxyModel", functionNames ) ]
-    if(length(ADproxyModel_functionNames) > 0) {
-        ## The following two headers were added to CPPincludes because nimDerives_dists must
-        ## come before Rmath.h
-        .self$CPPincludes <- c(nimbleIncludeFile("nimbleCppAD.h"),
-                               nimbleIncludeFile("nimDerivs_dists.h"), .self$CPPincludes)
-    }
-    for(fn in ADproxyModel_functionNames) {
-        thisDef <- .self$functionDefs[[fn]]
-        thisDef$returnType <- cppVarSym2templateTypeCppVarSym( thisDef$returnType,
-                                                              replacementBaseType = "CppAD::AD",
-                                                              replacementTemplateArgs = "double" )
-        parentST <- thisDef$code$objectDefs$getParentST()
-        thisDef$code$objectDefs <-
-            symbolTable2templateTypeSymbolTable(thisDef$code$objectDefs) #,
-##                                                replacementBaseType = "CppAD::AD",
-##                                                replacementTemplateArgs = "double" )
+  ## Update return type and names of functions like dnorm -> nimDerivs_dnorm
+  functionNames <- names(.self$functionDefs)
+  ADproxyModel_functionNames <- functionNames[ grepl("_ADproxyModel", functionNames ) ]
+  if(length(ADproxyModel_functionNames) > 0) {
+    ## The following two headers were added to CPPincludes because nimDerives_dists must
+    ## come before Rmath.h
+    .self$CPPincludes <- c(nimbleIncludeFile("nimbleCppAD.h"),
+                           nimbleIncludeFile("nimDerivs_dists.h"), .self$CPPincludes)
+  }
+
+  classST <- .self$objectDefs
+  classSymNames <- classST$getSymbolNames()
+  ADproxySymNames <- classSymNames[ grepl("ADproxyModel_", classSymNames ) ]
+  for(sn in ADproxySymNames) {
+    newSym <-
+      cppVarSym2templateTypeCppVarSym(classST$getSymbolObject(sn),
+                                      replacementBaseType = "CppAD::AD",
+                                      replacementTemplateArgs = "double") ## These end up the header file so they should not use TYPE_ as that is not typedef'd there
+    classST$addSymbol(newSym, allowReplace = TRUE)
+  }
+
+  for(fn in ADproxyModel_functionNames) {
+    thisDef <- .self$functionDefs[[fn]]
+    thisDef$returnType <- cppVarSym2templateTypeCppVarSym( thisDef$returnType,
+                                                          replacementBaseType = "CppAD::AD",
+                                                          replacementTemplateArgs = "double" )
+    parentST <- thisDef$code$objectDefs$getParentST()
+    thisDef$code$objectDefs <-
+      symbolTable2templateTypeSymbolTable(thisDef$code$objectDefs) #,
+    ##                                                replacementBaseType = "CppAD::AD",
+    ##                                                replacementTemplateArgs = "double" )
+    thisDef$code$objectDefs$setParentST(parentST)
+    thisDef$code$cppADCode <- 2L
+    ADtypeDefs <- symbolTable()
+    ADtypeDefs$addSymbol(cppVarFull(baseType = "typedef typename EigenTemplateTypes<CppAD::AD<double> >::typeEigenMapStrd", name = "EigenMapStrd_TYPE_") )
+    ADtypeDefs$addSymbol(cppVarFull(baseType = "typedef Matrix<CppAD::AD<double>, Dynamic, Dynamic>", name = "MatrixXd_TYPE_") )
+    ADtypeDefs$addSymbol(cppVarFull(baseType = "typedef CppAD::AD<double>", name = "TYPE_") )
+    thisDef$code$typeDefs <- ADtypeDefs
+    workEnv <- new.env()
+    workEnv$RsymTab <- thisDef$RCfunProc$compileInfo$newLocalSymTab
+    workEnv$.inModel <- TRUE
+    exprClasses_modifyForAD(thisDef$code$code, thisDef$code$objectDefs, workEnv)
+    if(isTRUE(workEnv$.illegalADcall)) {
+      handlingOption <- getNimbleOption("unsupportedDerivativeHandling")
+      if(identical(handlingOption, "ignore")) {
+        # do nothing
+      } else if(identical(handlingOption, "warn")) {
+        firstSym <- cppVarFull(name = "first", static = TRUE, baseType = "bool", constructor = "{true}")
+        thisDef$code$objectDefs <- symbolTable$new()
+        thisDef$code$typeDefs <- symbolTable$new()
         thisDef$code$objectDefs$setParentST(parentST)
-        thisDef$code$cppADCode <- 2L 
-        ADtypeDefs <- symbolTable()
-        ADtypeDefs$addSymbol(cppVarFull(baseType = "typedef typename EigenTemplateTypes<CppAD::AD<double> >::typeEigenMapStrd", name = "EigenMapStrd_TYPE_") )
-        ADtypeDefs$addSymbol(cppVarFull(baseType = "typedef Matrix<CppAD::AD<double>, Dynamic, Dynamic>", name = "MatrixXd_TYPE_") )
-        ADtypeDefs$addSymbol(cppVarFull(baseType = "typedef CppAD::AD<double>", name = "TYPE_") )
-        thisDef$code$typeDefs <- ADtypeDefs
-        workEnv <- new.env()
-        workEnv$RsymTab <- thisDef$RCfunProc$compileInfo$newLocalSymTab
-        workEnv$.inModel <- TRUE
-        exprClasses_modifyForAD(thisDef$code$code, thisDef$code$objectDefs, workEnv)
+        thisDef$code$objectDefs$addSymbol(firstSym)
+        errorTrappingCode <- quote({
+          if(first) {
+            nimPrint("\n  [Warning] In C++, a model node is being included in derivatives that does not support them.\n",
+                     "            This message will only be shown once. Results will not be valid.\n",
+                     "            To make this an error, set `nimbleOptions(unsupportedDerivativeHandling='error')`.\n",
+                     "            To skip this error handling entirely (and allow possible malformed code to be used),\n",
+                     "            set `nimbleOptions(unsupportedDerivativeHandling='ignore')`.")
+            first <- false
+          }
+          return(`CppAD::AD<double>`(0))
+        })
+        errorTrappingExpr <- RparseTree2ExprClasses(errorTrappingCode)
+        thisDef$code$code <- errorTrappingExpr
+      } else { #default to error
+        thisDef$code$objectDefs <- symbolTable$new()
+        thisDef$code$typeDefs <- symbolTable$new()
+        thisDef$code$objectDefs$setParentST(parentST)
+        msg <- paste0("\n  [Error] In C++, a model node is being included in derivatives that does not support them.\n",
+                     "          To make this a warning, set `nimbleOptions(unsupportedDerivativeHandling='warn')`.\n",
+                     "          To skip this error handling entirely (and allow possible malformed code to be used),\n",
+                     "          set `nimbleOptions(unsupportedDerivativeHandling='ignore')`.")
+        errorTrappingCode <- substitute(
+        {
+          nimStop(MSG)
+          return(`CppAD::AD<double>`(0))
+        }, list(MSG = msg))
+        errorTrappingExpr <- RparseTree2ExprClasses(errorTrappingCode)
+        thisDef$code$code <- errorTrappingExpr
+      }
     }
-    classST <- .self$objectDefs
-    classSymNames <- classST$getSymbolNames()
-    ADproxySymNames <- classSymNames[ grepl("ADproxyModel_", classSymNames ) ]
-    for(sn in ADproxySymNames) {
-        newSym <-
-            cppVarSym2templateTypeCppVarSym(classST$getSymbolObject(sn),
-                                            replacementBaseType = "CppAD::AD",
-                                            replacementTemplateArgs = "double") ## These end up the header file so they should not use TYPE_ as that is not typedef'd there
-        classST$addSymbol(newSym, allowReplace = TRUE)
-    }
-  
-    NULL
+  }
+  ## classST <- .self$objectDefs
+  ## classSymNames <- classST$getSymbolNames()
+  ## ADproxySymNames <- classSymNames[ grepl("ADproxyModel_", classSymNames ) ]
+  ## for(sn in ADproxySymNames) {
+  ##   newSym <-
+  ##     cppVarSym2templateTypeCppVarSym(classST$getSymbolObject(sn),
+  ##                                     replacementBaseType = "CppAD::AD",
+  ##                                     replacementTemplateArgs = "double") ## These end up the header file so they should not use TYPE_ as that is not typedef'd there
+  ##   classST$addSymbol(newSym, allowReplace = TRUE)
+  ## }
+  NULL
 }
 
 makeSingleCopyCall <- function(varName, cppCopyType,
