@@ -18,6 +18,7 @@ cppOutputCalls <- c(## makeCallList(recyclingRuleOperatorsAD, 'cppOutputRecyclin
                     list(nimDerivs_dummy = 'cppOutputNimDerivs'),
                     makeCallList(nimbleListReturningOperators, 'cppNimbleListReturningOperator'),
                     makeCallList(c("TFsetInput_", "TFgetOutput_", "TFrun_"), 'cppOutputMemberFunctionDeref'),
+                    makeCallList(c("stoch_ind_get", "stoch_ind_set"), "cppOutputStochIndGetSet"),
     list(
         'cwiseSqrt' = 'cppOutputEigMemberFunctionNoTranslate',
         'cwiseAbs' = 'cppOutputEigMemberFunctionNoTranslate_specialAD',
@@ -526,6 +527,14 @@ cppOutputBracket <- function(code, symTab) {
     paste0( nimGenerateCpp(code$args[[1]], symTab), brackets[1], paste0(unlist(lapply(code$args[-1], function(x) cppMinusOne(nimGenerateCpp(x, symTab) ) ) ), collapse = ', '), brackets[2] )
 }
 
+# Used for stoch_ind_get and stoch_ind_set, part of AD code only.
+cppOutputStochIndGetSet <- function(code, symTab) {
+  if(length(code$args) < 2) warning("There is a stochastic indexing call for AD with no indices.")
+    paste0(exprName2Cpp(code, symTab), '(',
+           nimGenerateCpp(code$args[[1]], symTab), ", ",
+           paste0(unlist(lapply(code$args[-1], function(x) cppMinusOne(nimGenerateCpp(x, symTab, asArg = TRUE) ) ) ), collapse = ', '), ')' )
+}
+
 cppOutputDoubleBracket <- function(code, symTab) {
     ## right now the only case is from a functionList, so we'll go straight there.
     cppOutputNimFunListAccess(code, symTab)
