@@ -29,6 +29,7 @@
 #include<limits>
 #include<string>
 #include<time.h>
+#include<cppad/cppad.hpp>
 using std::string;
 
 //using namespace std;
@@ -105,7 +106,7 @@ class nimbleTimerClass_ {
 
 #define ML_ERR_return_NAN { ML_ERROR(ME_DOMAIN, ""); return ML_NAN; }
 
-#define ML_ERROR(x, s) { \
+#define ML_ERROR(x, s) {                        \
    if(x > ME_DOMAIN) { \
        const char *msg = ""; \
        switch(x) { \
@@ -152,7 +153,8 @@ class nimbleTimerClass_ {
 	return R_D__0;					\
    }
 
-#define EIGEN_CHOL(x)       (x).selfadjointView<Eigen::Upper>().llt().matrixU()
+#define EIGEN_CHOL(x)       (x).template selfadjointView<Eigen::Upper>().llt().matrixU()
+#define nimDerivs_EIGEN_CHOL_no_atomic(x)       (x).template selfadjointView<Eigen::Upper>().llt().matrixU()
 //#define EIGEN_SOLVE(x,y)    (x).lu().solve(y)
 //#define EIGEN_FS(x,y)       (x).triangularView<Eigen::Lower>().solve(y)
 //#define EIGEN_BS(x,y)       (x).triangularView<Eigen::Upper>().solve(y)
@@ -167,30 +169,137 @@ bool nimNot(bool x);
 
 static inline bool isTRUE(bool x) {return(x);}
 
+inline double pow_int(double a, double b) {return pow(a, round(b));}
+inline double pow_int(double a, int b) {return pow(a, static_cast<double>(b));}
+
+// All the nimDerivs_FOO functions
+// were templated, but in a rush we
+// had to change some generated code to
+// std::function< CppAD::AD<double>(CppAD::AD<double) >( nimDerivs_FOO )
+// That does not do template type inference. And there are cases
+// where nimDerivs_FOO is not a template.
+// Quick solution is to remove the templating here, since
+// in practice T is always CppAD::AD<double>, and get that
+// hard-wired in.  There is an #undef at the end.
+#define T CppAD::AD<double>
+
 // needed for link functions
-double ilogit(double x);
-double icloglog(double x);
-double iprobit(double x);
-double probit(double x);
+inline double ilogit(double x) {return(1./(1. + exp(-x)));}
+//template<class T>
+inline T nimDerivs_ilogit(T x){
+  return(1./(1. + exp(-x)));
+}
+
+inline double icloglog(double x) {return(1.-exp(-exp(x)));}
+//template<class T>
+inline T nimDerivs_icloglog(T x){
+  return(1.-exp(-exp(x)));
+}
+
+inline double iprobit(double x) {return(pnorm(x, 0., 1., 1, 0));}
+inline double probit(double x) {return(qnorm(x, 0., 1., 1, 0));}
+
 //double abs(double x);
-double cloglog(double x);
-int nimEquals(double x1, double x2);
-double nimbleIfElse(bool condition, double x1, double x2);
-double lfactorial(double x);
-double factorial(double x);
+inline double cloglog(double x) {return(log(-log(1.-x)));}
+//template<class T>
+inline T nimDerivs_cloglog(T x){
+  return(log(-log(T(1) - x)));
+}
+
+inline int nimEquals(double x1, double x2) {return(x1 == x2 ? 1 : 0);}
+//template<class T>
+inline T nimDerivs_nimEquals(T x1, T x2){
+  return(CondExpEq(x1, x2, T(1), T(0)));
+}
+
+inline double nimbleIfElse(bool condition, double x1, double x2) {return(condition ? x1 : x2);}
+inline double lfactorial(double x) {return(lgammafn(x + 1));}
+inline double factorial(double x) {return(gammafn(1+x));}
 //double loggam(double x);
-double logit(double x);
-double nimRound(double x);
-double pairmax(double x1, double x2);
-double pairmin(double x1, double x2);
+inline double logit(double x) {return(log(x / (1.-x)));}
+//template<class T>
+inline T nimDerivs_logit(T x) {
+  return(log(x / (T(1)-x)));
+}
+
+inline double nimRound(double x) {return(round(x));}
+inline double pairmax(double x1, double x2) {return(x1 > x2 ? x1 : x2);}
+//template<class T>
+inline T nimDerivs_pairmax(T x1, T x2) {
+  return(CondExpGt(x1, x2, x1, x2));
+}
+
+inline double pairmin(double x1, double x2) {return(x1 < x2 ? x1 : x2);}
+//template<class T>
+inline T nimDerivs_pairmin(T x1, T x2) {
+  return(CondExpLt(x1, x2, x1, x2));
+}
+
 //double phi(double x);
-int nimStep(double x); 
-double cube(double x);
-double inprod(double v1, double v2);
+inline int nimStep(double x) { return(x >= 0 ? 1 : 0);}
+//template<class T>
+inline T nimDerivs_nimStep(T x){
+	return(CondExpGe(x, T(0), T(1), T(0)));
+}
+
+inline double cube(double x) {return(x*x*x);}
+//template<class T>
+inline T nimDerivs_cube(T x){
+	return(x*x*x);
+}
+
+inline double inprod(double v1, double v2) {return(v1*v2);}
+//template<class T>
+inline T nimDerivs_inprod(T v1, T v2) {
+  return(v1*v2);
+}
+
+//template<class T>
+inline T nimDerivs_atan(T x) {
+  return(CppAD::atan(x));
+}
+
+//template<class T>
+inline T nimDerivs_cosh(T x) {
+  return(CppAD::cosh(x));
+}
+
+//template<class T>
+inline T nimDerivs_sinh(T x) {
+  return(CppAD::sinh(x));
+}
+
+//template<class T>
+inline T nimDerivs_tanh(T x) {
+  return(CppAD::tanh(x));
+}
+
+//template<class T>
+inline T nimDerivs_acosh(T x) {
+  return(CppAD::acosh(x));
+}
+
+//template<class T>
+inline T nimDerivs_asinh(T x) {
+  return(CppAD::asinh(x));
+}
+
+//template<class T>
+inline T nimDerivs_atanh(T x) {
+  return(CppAD::atanh(x));
+}
+
+//template<class T>
+inline T nimDerivs_log1p(T x) {
+  return(CppAD::log1p(x));
+}
 
 inline double nimble_NaN() {
   return std::numeric_limits<double>::has_quiet_NaN
     ? std::numeric_limits<double>::quiet_NaN()
     : (0./0.);
 }
+
+#undef T
+
 #endif
