@@ -545,7 +545,7 @@ test_that("warnings for multiply-defined model nodes:", {
             mu ~ dnorm(mu0[i],1)
         }
     })
-    expect_warning(m <- nimbleModel(code), "'i' on the left-hand side of 'mu ~ ", fixed = TRUE)
+    expect_message(m <- nimbleModel(code), "Multiple definitions", fixed = TRUE)
     code <- nimbleCode({
         tmp ~ dnorm(0,1)
         for(i in 1:3) {
@@ -553,7 +553,7 @@ test_that("warnings for multiply-defined model nodes:", {
             mu ~ dnorm(0,1)
         }
     })
-    expect_warning(m <- nimbleModel(code), "'i' on the left-hand side of 'mu ~ ", fixed = TRUE)
+    expect_message(m <- nimbleModel(code), "Multiple definitions", fixed = TRUE)
     code <- nimbleCode({
         tmp ~ dnorm(0,1)
         for(i in 1:3) {
@@ -561,7 +561,7 @@ test_that("warnings for multiply-defined model nodes:", {
                 mu[i+2,1] ~ dnorm(0,1)
         }
     })
-    expect_warning(m <- nimbleModel(code), "'j' on the left-hand side of 'mu[i + 2, 1] ~ ", fixed = TRUE)
+    expect_message(m <- nimbleModel(code), "Multiple definitions", fixed = TRUE)
     code <- nimbleCode({
         tmp ~ dnorm(0,1)
         for(i in 1:3) {
@@ -570,7 +570,7 @@ test_that("warnings for multiply-defined model nodes:", {
                 mu[i+2,1,1] ~ dnorm(0,1)
         }
     })
-    expect_warning(m <- nimbleModel(code), "'j,k' on the left-hand side of 'mu[i + 2, 1, 1] ~ ", fixed = TRUE)
+    expect_message(m <- nimbleModel(code), "Multiple definitions", fixed = TRUE)
 })
 
 test_that("handling of missing indexes of expressions:", {
@@ -1010,5 +1010,24 @@ test_that("Example of splitVertices bug from Issue 1268 works.", {
   expect_no_error(Rmodel <- nimbleModel(code, constants = list(index=c(1,1))))
 })
 
+test_that("Reserved keywords as model vars are trapped.", {
+    code <- nimbleCode({
+        double ~ dnorm(0,1)
+    })
+    expect_error(m <- nimbleModel(code), "Found variable name that conflicts")
+})
+
+test_that("Warning printed when indexing info in user environment.", {
+    code <- nimbleCode({
+        for(i in 1:N)
+            y[i] ~ dnorm(0,1)
+    })
+    N <- 3
+    temporarilyAssignInGlobalEnv(N)
+    expect_message(m <- nimbleModel(code, constants = list(foo=3)),
+                   "Information has been found in the user's environment")
+})
+
+
 options(warn = RwarnLevel)
 nimbleOptions(verbose = nimbleVerboseSetting)
